#include <iostream>
#include <string>
#include <sstream>
#include <vector>
#include <curl/curl.h>
#include <stdio.h>
#include "histogram.h"
#include "text.h"
#include "svg.h"
using namespace std;

struct Input {
    vector<double> numbers;
    size_t bin_count{};
};

Input
input_data(istream& inp, bool prompt){
    size_t number_count;
    if (prompt){
        cerr << "enter number_count";
    }
    inp >> number_count;

    Input in;
    in.numbers.resize(number_count);
    if (prompt){
        std::cerr << "enter numbers";
    }
    for (size_t i = 0; i < number_count; i++){
        inp >> in.numbers[i];
    }
    if (prompt){
        std::cerr << "enter bin_count";
    }
    inp >> in.bin_count;
    return in;
}

size_t
write_data(void* items, size_t item_size, size_t item_count, void* ctx) {
    stringstream* buffer = reinterpret_cast<stringstream*>(ctx);
    size_t data_size = item_size * item_count;
    (*buffer).write(reinterpret_cast<const char*>(items), data_size);
    return data_size;
}

string
filling(const string& key, const string& color){
    if(strcmp(key.c_str(), "-fill") == 0){
        if (strcmp(color.c_str(), " ") == 0){
            cerr << "Error: no color specified";
            exit(1);
        }
        return color.c_str();
    }
}

Input
download(const string& adress){
    stringstream buffer;
    CURL* curl = curl_easy_init();
    if(curl) {
        CURLcode res;
        curl_easy_setopt(curl, CURLOPT_URL, adress.c_str());
        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_data);
        curl_easy_setopt(curl, CURLOPT_WRITEDATA, &buffer);
        res = curl_easy_perform(curl);
        if (res != CURLE_OK){
            cerr << curl_easy_strerror(res);
            exit(1);
        }
        curl_easy_cleanup(curl);
    }
    return input_data(buffer, false);
}

int main(int argc, char* argv[]){
    string color = "base";
    curl_global_init(CURL_GLOBAL_ALL);
    Input in;
    if (argc > 1) {
        in = download(argv[1]);
    } else {
        in = input_data(cin, true);
    }

    if (argc > 2){
        if (argv[3] == NULL){
            color = filling(argv[2], " ");
        }
        else{
        color = filling(argv[2], argv[3]);
        }
    }

    auto bins = make_histogram(in.numbers, in.bin_count);
    auto borders = make_borders(in.numbers, in.bin_count);
    show_histogram_svg(bins, borders, in.bin_count, color);
}
